/*
 * Copyright (C) 2006 Canonical Ltd.
 * Author: Alexander Sack <asac@jwsdot.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 * the full text of the license.
 */

#include <argp.h>
#include <error.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


const char *obscure_program_version =
"obscure 0.1";
const char *obscure_program_bug_address =
"Alexander Sack <asac@jwsdot.com>";
  
static char doc[] =
"obscure #1 -- a program shift a firefox configuration file by 'obscure_value'";

/* A description of the arguments we accept. */
static char args_doc[] = "INPUTFILE [OUTPUTFILE]";

static struct argp_option options[] = {
  {"offset",  'o', "INTEGER",      0,  "[DEFAULT=13] obscure value offset" },
  {"reverse",  'r', 0, 0,  "reverse direction of shift to unobscure" },
  {0}
};

struct arguments
{
  int obscure_value;
  int do_reverse;
  char *inputfile;
  char *outputfile;
};

static error_t
parse_opt (int key, char *arg, struct argp_state *state);

/* Our argp parser. */
static struct argp argp = { options, parse_opt, args_doc, doc };
    
static struct arguments arguments;

int main(int argc, char** argv)
{
  memset(&arguments, '\0', sizeof(struct arguments));

  arguments.obscure_value = 13;

  argp_parse (&argp, argc, argv, 0, 0, &arguments);

  if(arguments.do_reverse)
    arguments.obscure_value *= -1;

  int ifd = open(arguments.inputfile, O_RDONLY);
  int ofd = 1;

  if(arguments.outputfile) {
    ofd = open(arguments.outputfile, O_WRONLY | O_CREAT, 0644);
  }
  
  if(ifd == -1)
  {
    perror("cannot open inputfile");
  }
  if(ofd == -1)
  {
    perror("cannot open outputfile");
  }

  char buf[4096];
  int readn;
  while( (readn = read(ifd, buf, 4096)) > 0)
  {
    int writen=0;
    int i;

    for(i=0; i < readn; i++)
      buf[i]+=arguments.obscure_value;

    while(writen < readn)
       writen += write(ofd, buf, readn-writen);
  }

  return 0;
}


static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;
  switch (key)
  {
  case 'r':
    arguments->do_reverse = 1;
    break;
  case 'o':
    arguments->obscure_value = atoi(arg);
    break;
  case ARGP_KEY_ARG:
    if(state->arg_num == 0)
	arguments->inputfile = arg;
    else if(state->arg_num == 1)
	arguments->outputfile = arg;
    else
      argp_usage(state);
    break;
  case ARGP_KEY_END:
    if(state->arg_num < 1)
      argp_usage(state);
    break;
  default:
    return ARGP_ERR_UNKNOWN;
  }
  return 0;
}
